
#include "controller_fsm.h"

SC_MODULE(controller_fsm) {

sc_in<bool> clock, tb_read_rom, tb_read_rand, reset, read_rom_done;
sc_in<sc_lv<6> > Op;
sc_out<bool> pc_reset, PCWrite, PCInc, PCWriteCond, IorD, MemRead;
sc_out<bool> read_rom, MemWrite, IRWrite, MemtoReg;
sc_out<sc_uint<2> > PCSource;
sc_out<bool> TargetWrite;
sc_out<sc_uint<2> > ALUOp;
sc_out<sc_uint<2> > ALUSelB;
sc_out<bool> ALUSelA, RegWrite, RegDst;
fsm_state present_state;
fsm_state next_state;
scv_smart_ptr<fsm_state> ppresent_state;
scv_coverpoint cp_present_state;
scv_covergroup cg_present_state;

	SC_CTOR(controller_fsm): 
		clock("clock"),
		tb_read_rom("tb_read_rom"),
		tb_read_rand("tb_read_rand"),
		reset("reset"),
		read_rom_done("read_rom_done"),
		Op("Op"),
		pc_reset("pc_reset"),
		PCWrite("PCWrite"),
		PCInc("PCInc"),
		PCWriteCond("PCWriteCond"),
		IorD("IorD"),
		MemRead("MemRead"),
		read_rom("read_rom"),
		MemWrite("MemWrite"),
		IRWrite("IRWrite"),
		MemtoReg("MemtoReg"),
		PCSource("PCSource"),
		TargetWrite("TargetWrite"),
		ALUOp("ALUOp"),
		ALUSelB("ALUSelB"),
		ALUSelA("ALUSelA"),
		RegWrite("RegWrite"),
		RegDst("RegDst"),
		ppresent_state("present_state"),
		SCV_COVERPOINT_CTOR(cp_present_state),
		SCV_COVERPOINT_CTOR(cg_present_state) {

		SC_METHOD(ns_logic);
		sensitive_pos << clock;

		SC_METHOD(ps_logic);
		sensitive << clock;
//		cp_present_state.cover(scv_expression::create_reference(present_state),scv_coverbin::AUTO);
		cp_present_state.cover(ppresent_state,scv_coverbin::AUTO);
		cg_present_state.cover(cp_present_state);
		cg_present_state.sample_at(&clock);
	}


void ns_logic() {
	if(reset) {
		present_state = RESET;
	}
	else {
		present_state = next_state;
	}
ppresent_state.write(present_state);
}


void ps_logic() {
cve_coverage_save_nc();
switch(present_state) {
	case RESET:
		pc_reset.write(true);
		next_state=RESET;
		if(tb_read_rand == true) {
			next_state = READ_RAND;
		}
		else {
			next_state = READ_ROM;
		}
		break;
	case READ_ROM:
		pc_reset.write(true);
		MemWrite.write(true);
		read_rom.write(true);
		if(read_rom_done == true) {
			read_rom.write(false);
			next_state = INST_FETCH;
		}
		else {
			next_state = READ_ROM;
		}
		break;
	case READ_RAND:
		pc_reset.write(true);
		MemWrite.write(true);
		read_rom.write(false);
		if(tb_read_rand != true) {
			next_state = INST_FETCH;
		}
		else {
			next_state = READ_RAND;
		}
		break;
	case INST_FETCH: 
		pc_reset.write(false);
		next_state = INST_DECODE_REG_FETCH;
		PCWrite.write(false);
		PCInc.write(false);
		PCWriteCond.write(false);
		IorD.write(false);
		MemRead.write(true);
		MemWrite.write(false);
		IRWrite.write(true);
		MemtoReg.write(false);
		PCSource = 0;
		TargetWrite.write(false);
		ALUOp = 0;
		ALUSelB = 1;
		ALUSelA.write(false);
		RegWrite.write(false);
		RegDst.write(false);
		break;
        case INST_DECODE_REG_FETCH:
		pc_reset.write(false);
		PCWrite.write(false);
		PCInc.write(false);
		PCWriteCond.write(false);
		IorD.write(false);
		MemRead.write(false);
		MemWrite.write(false);
		IRWrite.write(false);
		MemtoReg.write(false);
		PCSource = 0;
		TargetWrite.write(true);
		ALUOp = 0;
		ALUSelB = 3;
		ALUSelA.write(false);
		RegWrite.write(false);
		RegDst.write(false);
		if(Op.read() == "100011" ) {
			next_state = MEM_ADDR_COMP;
		}
		if(Op.read() == "101011") {
			next_state = MEM_ADDR_COMP;
		}
		if(Op.read() == "000000") {
			next_state = EXECUTION;
		}
		if(Op.read() == "100100") {
			next_state = BRANCH_COMPLETE;
		}
		if(Op.read() == "000010") {
		next_state = JUMP_COMPLETE;
		}
		break;
	case MEM_ADDR_COMP:
		pc_reset.write(false);
		PCWrite.write(false);
		PCInc.write(false);
		PCWriteCond.write(false);
		IorD.write(false);
		MemRead.write(false);
		MemWrite.write(false);
		IRWrite.write(false);
		MemtoReg.write(false);
		PCSource = 0;
		TargetWrite.write(false);
		ALUOp = 0;
		ALUSelB = 2;
		ALUSelA.write(true);
		RegWrite.write(false);
		RegDst.write(false);
		if(Op.read() == "100011") {
			next_state = MEM_READ;
		}
		if(Op.read() == "101011") {
			next_state = MEM_WRITE;
		}
		break;
	case MEM_READ:
		pc_reset.write(false);
		PCWrite.write(false);
		PCInc.write(false);
		PCWriteCond.write(false);
		IorD.write(true);
		MemRead.write(true);
		MemWrite.write(false);
		IRWrite.write(false);
		MemtoReg.write(false);
		PCSource = 0;
		TargetWrite.write(false);
		ALUOp = 0;
		ALUSelB = 2;
		ALUSelA.write(true);
		RegWrite.write(false);
		RegDst.write(false);
		next_state = WRITE_BCK_STEP;
		break;
	case WRITE_BCK_STEP:
		pc_reset.write(false);
		PCWrite.write(false);
		PCInc.write(true);
		PCWriteCond.write(false);
		IorD.write(true);
		MemRead.write(true);
		MemWrite.write(false);
		IRWrite.write(false);
		MemtoReg.write(true);
		PCSource = 0;
		TargetWrite.write(false);
		ALUOp = 0;
		ALUSelB = 2;
		ALUSelA.write(true);
		RegWrite.write(true);
		RegDst.write(false);
		next_state = INST_FETCH;
		break;
	case MEM_WRITE:
		pc_reset.write(false);
		PCWrite.write(false);
		PCInc.write(true);
		PCWriteCond.write(false);
		IorD.write(true);
		MemRead.write(false);
		MemWrite.write(true);
		IRWrite.write(false);
		MemtoReg.write(false);
		PCSource = 0;
		TargetWrite.write(false);
		ALUOp = 0;
		ALUSelB = 2;
		ALUSelA.write(true);
		RegWrite.write(false);
		RegDst.write(false);
		next_state = INST_FETCH;
		break;
	case EXECUTION:
		pc_reset.write(false);
		PCWrite.write(false);
		PCInc.write(false);
		PCWriteCond.write(false);
		IorD.write(false);
		MemRead.write(false);
		MemWrite.write(false);
		IRWrite.write(false);
		MemtoReg.write(false);
		PCSource = 0;
		TargetWrite.write(false);
		ALUOp = 2;
		ALUSelB = 0;
		ALUSelA.write(true);
		RegWrite.write(false);
		RegDst.write(true);
		next_state = RTYPE_COMPLETE;
		break;
	case RTYPE_COMPLETE:
		pc_reset.write(false);
		PCWrite.write(false);
		PCInc.write(true);
		PCWriteCond.write(false);
		IorD.write(false);
		MemRead.write(false);
		MemWrite.write(false);
		IRWrite.write(false);
		MemtoReg.write(false);
		PCSource = 0;
		TargetWrite.write(false);
		ALUOp = 2;
		ALUSelB = 0;
		ALUSelA.write(true);
		RegWrite.write(true);
		RegDst.write(true);
		next_state = INST_FETCH;
		break;
	case BRANCH_COMPLETE:
		pc_reset.write(false);
		PCWrite.write(false);
		PCInc.write(true);
		PCWriteCond.write(true);
		IorD.write(false);
		MemRead.write(false);
		MemWrite.write(false);
		IRWrite.write(false);
		MemtoReg.write(false);
		PCSource = 1;
		TargetWrite.write(false);
		ALUOp = 1;
		ALUSelB = 0;
		ALUSelA.write(true);
		RegWrite.write(false);
		RegDst.write(false);
		next_state = INST_FETCH;
		break;
	case JUMP_COMPLETE:
		pc_reset.write(false);
		PCWrite.write(true);
		PCInc.write(false);
		PCWriteCond.write(false);
		IorD.write(false);
		MemRead.write(false);
		MemWrite.write(false);
		IRWrite.write(false);
		MemtoReg.write(false);
		PCSource = 2;
		TargetWrite.write(false);
		ALUOp = 0;
		ALUSelB = 0;
		ALUSelA.write(false);
		RegWrite.write(false);
		RegDst.write(false);
		next_state = INST_FETCH;
		break;
        default:
		next_state = present_state;
		break;
        }
}

};

//pragma coverage on

NCSC_MODULE_EXPORT(controller_fsm)


