library IEEE;
use IEEE.STD_Logic_1164.all;
use IEEE.STD_Logic_Unsigned.all;

entity alu is port(
			ALU_result: out std_logic_vector(31 downto 0);
			Zero: out std_logic;
			ALU_in1: in std_logic_vector(31 downto 0);
			ALU_in2: in std_logic_vector(31 downto 0);
			ALUOp: in std_logic_vector(1 downto 0);
			FCode: in std_logic_vector(5 downto 0)
			);
end entity alu;


architecture RTL of alu is

type FCode_TYPE is (ADD_T, SUB_T, AND_T, OR_T, SOL_T, X);

signal FCode_T : FCode_TYPE;

begin

F_CODE:
process (FCode)
begin
	case (FCode) is
		when "100000" =>
			FCode_T <= ADD_T;
		when "100010" =>
			FCode_T <= SUB_T;
		when "100100" =>
			FCode_T <= AND_T;
		when "100101" =>
			FCode_T <= OR_T;
		when "101010" =>
			FCode_T <= SOL_T;
                when others =>  FCode_T <= X;
end case;
end process F_CODE;

ALU_LOGIC:
process (ALU_in1, ALU_in2)
begin
	if(ALUOp = "00") then
		ALU_result <= ALU_in1 + ALU_in2;
		if((ALU_in1 + ALU_in2) = X"0000_0000") then
			Zero <= '1';
		else
			Zero <= '0';
		end if;
	elsif(ALUOp(0) = '1') then
		ALU_result <= ALU_in1 - ALU_in2;
		if((ALU_in1 - ALU_in2) = X"0000_0000") then
			Zero <= '1';
                else                                                                                                              
                        Zero <= '0';
		end if;
	elsif(ALUOp(1) = '1' AND FCode_T = ADD_T) then
		ALU_result <= ALU_in1 + ALU_in2;
		if((ALU_in1 + ALU_in2) = X"0000_0000") then
			Zero <= '1';
                else                                                                                                              
                        Zero <= '0';
		end if;
	elsif(ALUOp(1) = '1' AND FCode_T = SUB_T) then
		ALU_result <= ALU_in1 - ALU_in2;
		if((ALU_in1 - ALU_in2) = X"0000_0000") then
			Zero <= '1';
                else                                                                                                              
                        Zero <= '0';
		end if;
	elsif(ALUOp(1) = '1' AND FCode_T = AND_T) then
--		ALU_result <= ALU_in1 AND ALU_in2 AND X"0000_0000"; --com_test
		ALU_result <= ALU_in1 AND ALU_in2;
		if((ALU_in1 + ALU_in2) = X"0000_0000") then
			Zero <= '1';
                else                                                                                                              
                        Zero <= '0';
		end if;
	elsif(ALUOp(1) = '1' AND FCode_T = OR_T) then
--		ALU_result <= ALU_in1 or ALU_in2 or NOT(X"0000_0000"); --com_test
--		ALU_result <= ALU_in1 or ALU_in2 or X"FFFF_FFFF"; --com_test
		ALU_result <= ALU_in1 OR ALU_in2;
		if((ALU_in1 + ALU_in2) = X"0000_0000") then
			Zero <= '1';
                else                                                                                                              
                        Zero <= '0';
		end if;
	elsif(ALUOp(1) = '1' AND FCode_T = SOL_T) then
		if(ALU_in1 < ALU_in2) then
			ALU_result <= X"0000_0001";
                        Zero <= '0';
		else
			ALU_result <= X"0000_0000";
			Zero <= '1';
		end if;
	end if;
end process ALU_LOGIC;

end architecture RTL;

