library IEEE;
use IEEE.STD_Logic_1164.all;
use IEEE.Numeric_STD.all;


entity controller_fsm is port(
			clock, tb_read_rom, tb_read_rand, reset, read_rom_done: in std_logic;
			Op: in std_logic_vector(5 downto 0);
			pc_reset, PCWrite, PCInc, PCWriteCond, IorD, MemRead: out std_logic;
			read_rom, MemWrite, IRWrite, MemtoReg: out std_logic;
			PCSource: out std_logic_vector(1 downto 0);
			TargetWrite: out std_logic;
			ALUOp: out std_logic_vector(1 downto 0);
			ALUSelB: out std_logic_vector(1 downto 0);
			ALUSelA, RegWrite, RegDst: out std_logic);
end entity controller_fsm;


architecture RTL of controller_fsm is

type StateType is (RESET_S, READ_ROM_S, READ_RAND, INST_FETCH,
		INST_DECODE_REG_FETCH, MEM_ADDR_COMP, MEM_READ,
		WRITE_BCK_STEP,MEM_WRITE, EXECUTION,
		RTYPE_COMPLETE, BRANCH_COMPLETE, JUMP_COMPLETE);
type OpType is (RType, jmp, beq, lw, sw, X);

signal present_state, next_state : StateType;
signal OpCode : OpType;

begin

OP_CODE:
process (Op)
begin
	case (Op) is
		when "000000" =>
			OpCode <= RType;
		when "000010" =>
			OpCode <= jmp;
		when "100100" =>
			OpCode <= beq;
		when "100011" =>
			OpCode <= lw;
		when "101011" =>
			OpCode <= sw;
                when others =>  OpCode <= X;
end case;
end process OP_CODE;

PS_LOGIC:
process(clock)
begin
	case (present_state) is
		when RESET_S =>
				pc_reset <= '1';
				next_state <= RESET_S;
				if(tb_read_rand = '1') then
					next_state <= READ_RAND;
				end if;
				if(tb_read_rom = '1') then
					next_state <= READ_ROM_S;
				end if;
		when READ_ROM_S =>
				pc_reset <= '1';
				MemWrite <= '1';
				read_rom <= '1';
				if(read_rom_done = '1') then
					read_rom <= '0';
					next_state <= INST_FETCH;
				else
					next_state <= READ_ROM_S;
				end if;
		when READ_RAND =>
				pc_reset <= '1';
				MemWrite <= '1';
				read_rom <= '0';
				if(tb_read_rand /= '1') then
					next_state <= INST_FETCH;
				else
					next_state <= READ_RAND;
				end if;
		when INST_FETCH =>
				pc_reset <= '0';
				next_state <= INST_DECODE_REG_FETCH;
				PCWrite <= '0';
				PCInc <= '0';
				PCWriteCond <= '0';
				IorD <= '0';
				MemRead <= '1';
				MemWrite <= '0';
				IRWrite <= '1';
				MemtoReg <= '0';
				PCSource <= "00";
				TargetWrite <= '0';
				ALUOp <= "00";
				ALUSelB <= "01";
				ALUSelA <= '0';
				RegWrite <= '0';
				RegDst <= '0';
		when INST_DECODE_REG_FETCH =>
				pc_reset <= '0';
				PCWrite <= '0';
				PCInc <= '0';
				PCWriteCond <= '0';
				IorD <= '0';
				MemRead <= '0';
				MemWrite <= '0';
				IRWrite <= '0';
				MemtoReg <= '0';
				PCSource <= "00";
				TargetWrite <= '1';
				ALUOp <= "00";
				ALUSelB <= "11";
				ALUSelA <= '0';
				RegWrite <= '0';
				RegDst <= '0';
				if(OpCode = lw or OpCode = sw) then
					next_state <= MEM_ADDR_COMP;
				end if;
				if(OpCode = RType) then
					next_state <= EXECUTION;
				end if;
				if(OpCode = beq) then
					next_state <= BRANCH_COMPLETE;
				end if;
				if(OpCode = jmp) then
					next_state <= JUMP_COMPLETE;
				end if;
				if(tb_read_rand = '1') then
					next_state <= READ_RAND;
				end if;
				if(tb_read_rom = '1') then
					next_state <= READ_ROM_S;
				end if;
		when MEM_ADDR_COMP =>
				pc_reset <= '0';
				PCWrite <= '0';
				PCInc <= '0';
				PCWriteCond <= '0';
				IorD <= '0';
				MemRead <= '0';
				MemWrite <= '0';
				IRWrite <= '0';
				MemtoReg <= '0';
				PCSource <= "00";
				TargetWrite <= '0';
				ALUOp <= "00";
				ALUSelB <= "10";
				ALUSelA <= '1';
				RegWrite <= '0';
				RegDst <= '0';
				if(OpCode = lw) then
					next_state <= MEM_READ;
				end if;
				if(OpCode = sw) then
					next_state <= MEM_WRITE;
				end if;
		when MEM_READ =>
				pc_reset <= '0';
				PCWrite <= '0';
				PCInc <= '0';
				PCWriteCond <= '0';
				IorD <= '1';
				MemRead <= '1';
				MemWrite <= '0';
				IRWrite <= '0';
				MemtoReg <= '0';
				PCSource <= "00";
				TargetWrite <= '0';
				ALUOp <= "00";
				ALUSelB <= "10";
				ALUSelA <= '1';
				RegWrite <= '0';
				RegDst <= '0';
				next_state <= WRITE_BCK_STEP;
		when WRITE_BCK_STEP =>
				pc_reset <= '0';
				PCWrite <= '0';
				PCInc <= '1';
				PCWriteCond <= '0';
				IorD <= '1';
				MemRead <= '1';
				MemWrite <= '0';
				IRWrite <= '0';
				MemtoReg <= '1';
				PCSource <= "00";
				TargetWrite <= '0';
				ALUOp <= "00";
				ALUSelB <= "10";
				ALUSelA <= '1';
				RegWrite <= '1';
				RegDst <= '0';
				next_state <= INST_FETCH;
		when MEM_WRITE =>
				pc_reset <= '0';
				PCWrite <= '0';
				PCInc <= '1';
				PCWriteCond <= '0';
				IorD <= '1';
				MemRead <= '0';
				MemWrite <= '1';
				IRWrite <= '0';
				MemtoReg <= '0';
				PCSource <= "00";
				TargetWrite <= '0';
				ALUOp <= "00";
				ALUSelB <= "10";
				ALUSelA <= '1';
				RegWrite <= '0';
				RegDst <= '0';
				next_state <= INST_FETCH;
		when EXECUTION =>
				pc_reset <= '0';
				PCWrite <= '0';
				PCInc <= '0';
				PCWriteCond <= '0';
				IorD <= '0';
				MemRead <= '0';
				MemWrite <= '0';
				IRWrite <= '0';
				MemtoReg <= '0';
				PCSource <= "00";
				TargetWrite <= '0';
				ALUOp <= "10";
				ALUSelB <= "00";
				ALUSelA <= '1';
				RegWrite <= '0';
				RegDst <= '1';
				next_state <= RTYPE_COMPLETE;			
		when RTYPE_COMPLETE =>
				pc_reset <= '0';
				PCWrite <= '0';
				PCInc <= '1';
				PCWriteCond <= '0';
				IorD <= '0';
				MemRead <= '0';
				MemWrite <= '0';
				IRWrite <= '0';
				MemtoReg <= '0';
				PCSource <= "00";
				TargetWrite <= '0';
				ALUOp <= "10";
				ALUSelB <= "00";
				ALUSelA <= '1';
				RegWrite <= '1';
				RegDst <= '1';
				next_state <= INST_FETCH;
		when BRANCH_COMPLETE =>
				pc_reset <= '0';
				PCWrite <= '0';
				PCInc <= '1';
				PCWriteCond <= '1';
				IorD <= '0';
				MemRead <= '0';
				MemWrite <= '0';
				IRWrite <= '0';
				MemtoReg <= '0';
				PCSource <= "01";
				TargetWrite <= '0';
				ALUOp <= "01";
				ALUSelB <= "00";
				ALUSelA <= '1';
				RegWrite <= '0';
				RegDst <= '0';
				next_state <= INST_FETCH;
		when JUMP_COMPLETE =>
				pc_reset <= '0';
				PCWrite <= '1';
				PCInc <= '0';
				PCWriteCond <= '0';
				IorD <= '0';
				MemRead <= '0';
				MemWrite <= '0';
				IRWrite <= '0';
				MemtoReg <= '0';
				PCSource <= "10";
				TargetWrite <= '0';
				ALUOp <= "00";
				ALUSelB <= "00";
				ALUSelA <= '0';
				RegWrite <= '0';
				RegDst <= '0';
				next_state <= INST_FETCH;
		when others =>	next_state <= present_state;
end case;
end process PS_LOGIC;


NS_LOGIC:
process(clock)
begin
if rising_edge(clock) then
	if(reset = '1') then
		present_state <= RESET_S;
	else
		present_state <= next_state;
	end if;
end if;
end process NS_LOGIC;

end architecture RTL;

