
//cpu
//Eric R. Williamson
//JUL2005


`timescale 1ns / 1ps

module cpu ( clock, tb_read_rom, tb_read_rand, reset, random_inst );

input clock;
input tb_read_rom, tb_read_rand, reset;
input [31:0] random_inst;

// data path signals
wire [31:0] pc;
wire [31:0] ReadAddress;
wire [31:0] WriteAddress;
wire [31:0] WriteData;
wire [31:0] mem_data;
wire [5:0] op;
wire [25:0] i_lower;
wire [4:0] write_reg;
wire [31:0] write_reg_data;
wire [31:0] sign_extend_out;
wire [31:0] read_data2;
wire [31:0] read_data1;
wire [31:0] alu_in1;
wire [31:0] alu_in2;
wire [31:0] alu_result;
wire [31:0] target;
wire [31:0] feedback;

// control path signals
wire PCInc;
wire PCWrite;
wire PCWriteCond;
wire IorD;
wire MemRead;
wire MemWrite;
wire IRWrite;
wire MemtoReg;
wire [1:0] PCSource;
wire TargetWrite;
wire [1:0] ALUOp;
wire [1:0] ALUSelB;
wire ALUSelA;
wire RegWrite;
wire RegDst;

wire Zero;
wire and_out;
wire or_out;

wire [31:0] rom_data;
wire [7:0] rom_address;
wire [7:0] ram_address;

wire read_rom_done;
wire read_rom;
wire pc_reset;
wire read_rom_or;


// GATE INSTANCES

one_hot u101(
		.a(Zero),
		.rst(reset),
		.clock(clock)
);

pc u1(
	.clock(clock),
	.pc_reset(pc_reset),
	.PCInc(PCInc),
	.PCWrite(or_out),
	.PC_wr_data(feedback),
	.PC_rd_data(pc)
);

mux32 u2(
	.out(ReadAddress),
	.a(pc),
	.b(alu_result),
	.sel(IorD)
);

rom u19(
	.clock(clock),
	.read_rom(read_rom),
	.read_rom_done(read_rom_done ),
	.data_out(rom_data),
	.rom_address(rom_address)
);

ram_addr_cnt u23(
	.count(ram_address),
	.clock(clock),
	.reset(reset)
);

mux421a u20(
	.out(WriteAddress),
	.a(alu_result),
	.b(ram_address),
	.c(rom_address),
	.d(rom_address),
	.sel({read_rom_or, tb_read_rand})
);

mux421 u21(
	.out(WriteData),
	.a(read_data2),
	.b(random_inst),
	.c(rom_data),
	.d(rom_data),
	.sel({read_rom_or, tb_read_rand})
);

or u22(read_rom_or, read_rom, tb_read_rom);

ram u3(
	.clock(clock),
	.ReadAddress(ReadAddress),
	.WriteAddress(WriteAddress),
	.WriteData(WriteData),
	.MemRead(MemRead),
	.MemWrite(MemWrite),
	.MemData(mem_data)
);

ir u4(
	.clock(clock),
	.IRWrite(IRWrite),
	.wr_data(mem_data),
	.rd_data_upper(op),
       	.rd_data_lower(i_lower)
);

mux5 u5(
	.out(write_reg),
	.a(i_lower[20:16]),
	.b(i_lower[15:11]),
	.sel(RegDst)
);

mux32 u6(
	.out(write_reg_data),
	.a(alu_result),
	.b(mem_data),
	.sel(MemtoReg)
);

registers u7(
	.clock(clock),
	.RegWrite(RegWrite),
	.rd_reg1(i_lower[25:21]),
	.rd_reg2(i_lower[20:16]),
	.wr_reg(write_reg),
	.wr_data(write_reg_data),
	.rd_data1(read_data1),
	.rd_data2(read_data2)
);

sign_ext u8(
	.sign_ext_out(sign_extend_out),
	.sign_ext_in(i_lower[15:0])
);

mux32 u10(
	.out(alu_in1),
	.a(pc),
	.b(read_data1),
	.sel(ALUSelA)
);

mux421 u11(
	.out(alu_in2),
	.a(read_data2),
	.b(0),
	.c(sign_extend_out),
	.d(sign_extend_out),
	.sel(ALUSelB)
);

alu u12(
	.ALU_result(alu_result),
	.Zero(Zero),
	.ALU_in1(alu_in1),
	.ALU_in2(alu_in2),
	.ALUOp(ALUOp),
	.FCode(i_lower[5:0])
);

target u14(
	.clock(clock),
	.TWrite(TargetWrite),
	.t_data(alu_result),
	.rd_data(target)
);

mux421 u15(
	.out(feedback),
	.a(alu_result),
	.b(target),
	.c({pc[31:26], i_lower}),
	.d(),
	.sel(PCSource)
);

controller_fsm u16(
	.clock(clock),
	.tb_read_rom(tb_read_rom),
	.tb_read_rand(tb_read_rand),
	.reset(reset),
	.read_rom(read_rom),
	.read_rom_done(read_rom_done),
	.Op(op),
	.pc_reset(pc_reset),
	.PCWrite(PCWrite),
	.PCInc(PCInc),
	.PCWriteCond(PCWriteCond),
	.IorD(IorD),
	.MemRead(MemRead),
	.MemWrite(MemWrite),
	.IRWrite(IRWrite),
	.MemtoReg(MemtoReg),
	.PCSource(PCSource),
	.TargetWrite(TargetWrite),
	.ALUOp(ALUOp),
	.ALUSelB(ALUSelB),
	.ALUSelA(ALUSelA),
	.RegWrite(RegWrite),
	.RegDst(RegDst)
);

and u17 (and_out, PCWriteCond, Zero);
or u18 (or_out, and_out, PCWrite);

endmodule


