
`timescale 1 ns / 1 ps

module ram ( clock, ReadAddress, WriteAddress, WriteData, MemRead, MemWrite, MemData ) ;

inout [31:0] ReadAddress;
input [31:0] WriteAddress;
input [31:0] WriteData;
input clock, MemRead, MemWrite;
output [31:0] MemData;

reg [31:0] memory [0:255];
reg [31:0] MemData;

reg [7:0] addr_reg;
reg [5:0] op_reg;

//RType
reg [4:0] rtype_rs, rtype_rt, rtype_rd, rtype_shamt;
reg [5:0] rtype_funct;

//IType
reg [4:0] itype_rs, itype_rt;
reg [15:0] itype_immed;

//JType
reg [25:0] jtype_target;

//RType
// | op 6bits | rs 5bits | rt 5bits | rd 5bits | shamt 5bits | funct 6bits |

//IType
// | op 6bits | rs 5bits | rt 5bits | address/immediate 16bits |

//JType
// | op 6bits | target address 26 bits |

always @(posedge clock) fork
assign addr_reg = WriteAddress[7:0];
assign op_reg = MemData[31:26];

assign rtype_rs = MemData[25:21];
assign rtype_rt = MemData[20:16];
assign rtype_rd = MemData[15:11];
assign rtype_shamt = MemData[10:6];
assign rtype_funct = MemData[5:0];

assign itype_rs= MemData[25:21];
assign itype_rt = WriteAddress[20:16];
assign itype_immed = MemData[15:0];

assign jtype_target = MemData[25:0];

join


//always @(posedge clock) begin
//$display("addr_lower_reg=%d",addr_lower_reg);
//end


//write
always @(posedge clock) begin
	if(MemWrite)
		memory [WriteAddress[7:0]] = WriteData;
	assign MemData = MemRead ? memory [ReadAddress[7:0]] : 'bz;
end


task mem_dump;
	input [31:0] low, high;
	integer i;
	begin
		for( i=low; i<=high; i=i+1 )
				$display("memory[%d] = %d", i, memory[i]);
//	$stop;
	end
endtask

task mem_dump_stop;
	input [31:0] low, high;
	integer i;
	begin
		for( i=low; i<=high; i=i+1 )
				$display("memory[%d] = %d", i, memory[i]);
	$stop;
	end
endtask

endmodule

