
`timescale 1 ns / 1 ps

module ram ( clock, ReadAddress, WriteAddress, WriteData, MemRead, MemWrite, MemData ) ;

inout [31:0] ReadAddress;
input [31:0] WriteAddress;
input [31:0] WriteData;
input clock, MemRead, MemWrite;
output [31:0] MemData;

reg [31:0] memory [0:255];
reg [31:0] MemData;

reg [7:0] addr_reg;
reg [5:0] op_reg;

//RType
reg [4:0] rtype_rs, rtype_rt, rtype_rd, rtype_shamt;
reg [5:0] rtype_funct;

//IType
reg [4:0] itype_rs, itype_rt;
reg [15:0] itype_immed;

//JType
reg [25:0] jtype_target;

//RType
// | op 6bits | rs 5bits | rt 5bits | rd 5bits | shamt 5bits | funct 6bits |

//IType
// | op 6bits | rs 5bits | rt 5bits | address/immediate 16bits |

//JType
// | op 6bits | target address 26 bits |

always @(posedge clock) fork
assign addr_reg = WriteAddress[7:0];
assign op_reg = MemData[31:26];

assign rtype_rs = MemData[25:21];
assign rtype_rt = MemData[20:16];
assign rtype_rd = MemData[15:11];
assign rtype_shamt = MemData[10:6];
assign rtype_funct = MemData[5:0];

assign itype_rs= MemData[25:21];
assign itype_rt = WriteAddress[20:16];
assign itype_immed = MemData[15:0];

assign jtype_target = MemData[25:0];

join


//always @(posedge clock) begin
//$display("addr_lower_reg=%d",addr_lower_reg);
//end

covergroup cg @(posedge clock);
ADDR: coverpoint addr_reg {
	bins addr_b[] = {[1:255]}; // iff ^addr_upper_reg;
	bins addr_zero[] = {0}; //zero happens a lot and we want a better bar graph
}
OP: coverpoint op_reg {
	bins op_b[] = {[35:36],43,2};
	bins op_zero[] = {0};
}
R_RS: coverpoint rtype_rs {
	bins r_rs_b[] = {[0:31]}; // iff ^addr_upper_reg;
}
R_RT: coverpoint rtype_rt {
	bins r_rt_b[] = {[0:31]};
}
R_RD: coverpoint rtype_rd {
	bins r_rd_b[] = {[0:31]};
}
R_SHAMT: coverpoint rtype_shamt {
	bins r_shamt_b[] = {[1:31]};
	bins r_shamt_zero[] = {0};
}
R_FUNCT: coverpoint rtype_funct {
	bins rtype_funct_b[] = {32, 34, 36, 37, 42}; //ADD=32,SUB=34,AND=36,OR=37,SOL=42
}
I_RS: coverpoint itype_rs {
	bins i_rs_b[] = {[0:31]}; // iff ^addr_upper_reg;
}
I_RT: coverpoint itype_rt {
	bins i_rt_b[] = {[0:31]};
}
I_IMMED: coverpoint itype_immed {
	bins i_immed_b[] = {[0:65535]}; // iff ^addr_upper_reg;
}
J_TARGET: coverpoint jtype_target {
	bins j_target_b[] = {[0:67108863]};
}

//option.cross_auto_bin_max = 20;
CROSS_LABEL: cross OP, J_TARGET;
//CROSS_INSANE: cross ADDR, OP, R_RS, R_RT, R_RD, R_SHAMT, R_FUNCT, I_RS, I_RT, I_IMMED, J_TARGET; 
endgroup

cg cg_inst = new();


//write
always @(posedge clock) begin
	if(MemWrite)
		memory [WriteAddress[7:0]] = WriteData;
	assign MemData = MemRead ? memory [ReadAddress[7:0]] : 'bz;
end


task mem_dump;
	input [31:0] low, high;
	integer i;
	begin
		for( i=low; i<=high; i=i+1 )
				$display("memory[%d] = %d", i, memory[i]);
//	$stop;
	end
endtask

task mem_dump_stop;
	input [31:0] low, high;
	integer i;
	begin
		for( i=low; i<=high; i=i+1 )
				$display("memory[%d] = %d", i, memory[i]);
	$stop;
	end
endtask

endmodule

