
//-----------------This testbench has the following constructs-------
//
//	Program Block (pb)
//	Instantiating DUT in program block not allowed
//	Built in final block makes use of program block impossible
//	given limitation of not being able to instantiate DUT's in
//	program block
//
//	Single form of unpacked array not supported
//	Did not implement example of SysVlog arrays
//	given the time allowed
//
//	Final Block
//	Interfaces
//	data Classes (cl, clx means extended)
//
//	Packages
//	can't have classes in package, so did not use packages for this design
//
//	Coverage
//
//	No clocking block.
//	Could not get clocking blocks to work in a useful way
//
//	Randomized data: RnC in a class
//	More Hungarian Notation:
//	I1, I2, Instances
//	i, int
//	r, reg
//-----------------


interface tb_in(output reg clock, reset, read_rom, read_rand);

reg [31:0] rand_inst;
int success_i;
typedef enum {RType, IType, IType_sw, JType} InstType_e;
typedef enum {ADD, SUB, AND, OR, SOL} FuncType_e;

class Instruction_cl;
rand bit [5:0] op;
rand bit [4:0] rs;
rand bit [4:0] rt;
rand bit [4:0] rd;
rand bit [4:0] shamt;
rand bit [5:0] funct;
rand bit [15:0] immed;
rand bit [25:0] target;
	function new (integer seed);
                this.srandom(seed);
        endfunction
endclass

class Instruction_clx1 extends Instruction_cl;
rand InstType_e InstType_e_I1;
rand FuncType_e FuncType_e_I1;
constraint constraint_I1 {InstType_e_I1 dist {RType := 80, IType := 10, JType := 10};}
constraint constraint_I2
        {
                (InstType_e_I1 == RType ) -> op inside { [0 : 0] };
                (InstType_e_I1 == IType ) -> op inside { [35 : 36],43 };
                (InstType_e_I1 == JType ) -> op inside { [2 : 2] };
        }
constraint constraint_I3 {FuncType_e_I1 dist {ADD := 22, SUB := 22, AND := 22, OR := 22, SOL := 12};}
constraint constraint_I4
        {
                (FuncType_e_I1 == ADD ) -> funct inside { [32 : 32] };
                (FuncType_e_I1 == SUB ) -> funct inside { [34 : 34] };
		(FuncType_e_I1 == AND) -> funct inside { [36 : 36] };
		(FuncType_e_I1 == OR) -> funct inside { [37 : 37] };
		(FuncType_e_I1 == SOL ) -> funct inside { [42 : 42] };
        }
constraint constraint_I5
	{
		target inside { [0 : 255] };
	}
	function new (integer seed);
                super.new(seed);
        endfunction
endclass

// SYSTEMVERILOG: package class instantiation
Instruction_clx1 Instruction_clx1_I1 = new(6895);

//RType
// | op 6bits | rs 5bits | rt 5bits | rd 5bits | shamt 5bits | funct 6bits |

//IType
// | op 6bits | rs 5bits | rt 5bits | address/immediate 16bits |

//JType
// | op 6bits | target address 26 bits |


always @clock begin
success_i = Instruction_clx1_I1.randomize();
        case(Instruction_clx1_I1.InstType_e_I1)
                RType: begin
rand_inst = {Instruction_clx1_I1.op,Instruction_clx1_I1.rs,Instruction_clx1_I1.rt,Instruction_clx1_I1.rd,Instruction_clx1_I1.shamt,Instruction_clx1_I1.funct};
                end
                IType: begin
rand_inst = {Instruction_clx1_I1.op,Instruction_clx1_I1.rs,Instruction_clx1_I1.rt,Instruction_clx1_I1.immed};
                end
                JType: begin
rand_inst = {Instruction_clx1_I1.op,Instruction_clx1_I1.target};
                end
        endcase
end


initial begin : READ_ROM
	#50 read_rom = 1'b0;
	#350 read_rom = 1'b1;
	#450 read_rom = 1'b0;
end

initial begin : READ_RAND
	#50 read_rand = 1'b0;
	#58000 read_rand = 1'b1;
	#62000 read_rand = 1'b0;
end

initial begin : RESET
	#50 reset = 1'b0;
	#50 reset = 1'b1;
	#150 reset = 1'b0;
end

always begin : CLOCK
	#50 clock = 1'b0;
	#50 clock = 1'b1;
end

initial #80000 $finish;

final begin: END
$display("--------------Contents of Registers-----------------------");
$display("----------Feeding in Constrained Random Values and ROM data-------");
TB3.DUT.u7.mem_dump( 0, 31 );
end

endinterface



module TB3 ();

// SYSTEMVERILOG: interfaces
tb_in tb_in_I1 (clock, reset, read_rom, read_rand );


cpu DUT                 (       .clock(clock),
                                .tb_read_rom(read_rom),
                                .tb_read_rand(read_rand),
				.reset(reset),
                                .random_inst(tb_in_I1.rand_inst)
                        );

endmodule

